const std = @import("std");
const stdout = std.io.getStdOut().writer();

const InteractionError = error{
    TaskNotFound,
};

pub fn userInteraction(allocator: std.mem.Allocator, tasks: *std.ArrayList([]const u8)) !void {
    var failure: usize = 0;

    while (failure < 3) {
        try clearTerminal();
        printItems(tasks) catch |err| {
            std.debug.print("Failed to print the items:\n{}\n", .{err});
            failure += 1;
            continue;
        };

        const Case = enum { add, edit, remove, quit };

        try stdout.print("Please enter your choice [add/remove/edit/quit]: ", .{});
        const input = getInput(allocator) catch |err| {
            std.debug.print("Error acquiring input:\n{}\n", .{err});
            continue;
        };
        defer allocator.free(input);

        const case = std.meta.stringToEnum(Case, input) orelse continue;

        var task_name: []const u8 = undefined;

        if (case != .quit) {
            try stdout.print("Please enter task name: ", .{});
            task_name = getInput(allocator) catch |err| {
                std.debug.print("Error acquiring input:\n{}\n", .{err});
                continue;
            };
        }

        switch (case) {
            .add => {
                defer allocator.free(task_name);
                const duped = try allocator.dupe(u8, task_name);
                tasks.append(duped) catch |err| {
                    std.debug.print("Failed to add the specified task:\n{}\n", .{err});
                };
            },
            .edit => {
                defer allocator.free(task_name);
                const id = getTaskIndex(tasks, task_name) catch {
                    std.debug.print("Invalid task name...", .{});
                    continue;
                };
                try stdout.print("Please enter new task name: ", .{});
                const new_task_name = try getInput(allocator);
                defer allocator.free(new_task_name);

                const new_task_item = try allocator.dupe(u8, new_task_name);
                // free existing memory alloc
                allocator.free(tasks.items[id]);
                tasks.items[id] = new_task_item;
            },
            .remove => {
                defer allocator.free(task_name);
                const id = getTaskIndex(tasks, task_name) catch {
                    std.debug.print("Invalid task name...", .{});
                    continue;
                };

                // free existing memory alloc
                allocator.free(tasks.items[id]);
                _ = tasks.orderedRemove(id);
            },
            .quit => break,
        }
    }
}

fn getInput(allocator: std.mem.Allocator) ![]const u8 {
    const stdin = std.io.getStdIn().reader();
    const bare_line = try stdin.readUntilDelimiterAlloc(allocator, '\n', 8192);
    // defer std.heap.page_allocator.free(bare_line);
    // windows legacy
    return std.mem.trim(u8, bare_line, "\r");
}

fn getTaskIndex(tasks: *std.ArrayList([]const u8), task_name: []const u8) !usize {
    if (tasks.items.len == 0) return InteractionError.TaskNotFound;
    const task_id = std.fmt.parseInt(usize, task_name, 10) catch {
        for (tasks.items, 0..) |value, i| {
            if (std.mem.eql(u8, task_name, value)) {
                return i;
            }
        }

        return InteractionError.TaskNotFound;
    };

    const act_val = task_id - 1;
    if (act_val >= 0 and act_val < tasks.items.len) return act_val;

    return InteractionError.TaskNotFound;
}

fn printItems(tasks: *std.ArrayList([]const u8)) !void {
    if (tasks.items.len <= 0) {
        try stdout.print("No items\n---------\n", .{});
        return;
    }

    try stdout.print("Items:\n------\n", .{});
    for (tasks.items, 1..) |value, i| {
        try stdout.print("{d}. {s}\n", .{i, value});
    }
    try stdout.print("\n", .{});
}

fn clearTerminal() !void {
    // clear terminal
    try stdout.print("\x1B[2J\x1B[H", .{});
}
